<?php

namespace Sonicx\Core\Cache;

use http\Exception\RuntimeException;
use PHPMailer\PHPMailer\Exception;
use Sonicx\Core\Storage\Filesystem;
use Sonicx\Core\Context\Context;
use Sonicx\Core\REST\HTTPClient;
use Sonicx\Core\Device\Device;
use Sonicx\Core\Device\DeviceType;

$sonicx_basePath = dirname(__FILE__) . '/../';
require_once $sonicx_basePath . 'includes/constant.php';
require_once $sonicx_basePath . 'includes/filesystem.php';
require_once $sonicx_basePath . 'includes/context.php';
require_once $sonicx_basePath . 'includes/HTTPClient.php';
require_once $sonicx_basePath . 'includes/Device.php';

class Cache
{

    public function synchronizeCache() {
        $configuration = Context::getContext()->getConfiguration();
        $websiteId = $configuration[SONICX_WEBSITE_ID_KEY];
        $websiteSecret = $configuration[SONICX_WEBSITE_SECRET_KEY];
        $params = [SONICX_WEBSITE_ID_KEY => $websiteId, SONICX_WEBSITE_SECRET_KEY => $websiteSecret];
        $cacheData = HTTPClient::sendGETRequest(SONICX_WEBSITE_ACTIVITY_PURGE_URL, $params);
        $cacheDataUpdate = [];
        foreach($cacheData as $value) {
            $mobileCachedVersionId = $value[OPTIMIZATION_KEY][MOBILE_PLATFORM_KEY][OPTIMIZATION_VERSION_ID_KEY];
            $mobileCachedVersionIdCached = $value[OPTIMIZATION_KEY][MOBILE_PLATFORM_KEY][OPTIMIZATION_VERSION_ID_CACHED_KEY];
            $desktopCachedVersionId = $value[OPTIMIZATION_KEY][DESKTOP_PLATFORM_KEY][OPTIMIZATION_VERSION_ID_KEY];
            $desktopCachedVersionIdCached = $value[OPTIMIZATION_KEY][DESKTOP_PLATFORM_KEY][OPTIMIZATION_VERSION_ID_CACHED_KEY];
            $optimizationData = [URL_KEY => $value[URL_KEY]];
            if($mobileCachedVersionId != $mobileCachedVersionIdCached) {
                $optimizationData["mobileOptimizedUrl"] = $value[OPTIMIZATION_KEY][MOBILE_PLATFORM_KEY][OPTIMIZATION_URL_KEY];
            }
            if($desktopCachedVersionId != $desktopCachedVersionIdCached) {
                $optimizationData["desktopOptimizedUrl"] = $value[OPTIMIZATION_KEY][DESKTOP_PLATFORM_KEY][OPTIMIZATION_URL_KEY];
            }
            $this->rebuildPageCache($optimizationData);
            if($mobileCachedVersionId != $mobileCachedVersionIdCached) {
                $cacheDataUpdateStatusMobile = [WEBPAGE_ID_KEY => $value[WEBPAGE_ID_KEY], URL_KEY => $value[URL_KEY], PLATFORM_KEY => MOBILE_PLATFORM_KEY, OPTIMIZATION_VERSION_ID_CACHED_KEY => $mobileCachedVersionId];
                array_push($cacheDataUpdate, $cacheDataUpdateStatusMobile);
            }
            if($desktopCachedVersionId != $desktopCachedVersionIdCached) {
                $cacheDataUpdateStatusDesktop = [WEBPAGE_ID_KEY => $value[WEBPAGE_ID_KEY], URL_KEY => $value[URL_KEY], PLATFORM_KEY => DESKTOP_PLATFORM_KEY, OPTIMIZATION_VERSION_ID_CACHED_KEY => $desktopCachedVersionId];
                array_push($cacheDataUpdate, $cacheDataUpdateStatusDesktop);
            }
        }
        HTTPClient::sendPOSTRequest(SONICX_WEBPAGE_CACHE_URL, ["webpageCacheStatuses" => $cacheDataUpdate]);
        return true;
    }
    public function getPageCache($url, $userAgent)
    {
        $this->isValidURL();
        $configuration = Context::getContext()->getConfiguration();
        $siteId = $configuration[SONICX_WEBSITE_ID_KEY];
        $websiteCacheFolder = SONICX_CACHE_PATH . SONICX_CACHE_FOLDER_DELIMITER . $siteId;
        $websitePageCacheFolder = $websiteCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . md5($url);
        $device = new Device($userAgent);
        if ($this->isCacheValid($siteId, $url, $userAgent)) {
            try {
                if ($device->getType() == DeviceType::MOBILE) {
                    $file = $websitePageCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . SONICX_CACHE_MOBILE_SITE_NAME . SONICX_CACHE_SITE_HTML_EXTENSION;
                    return $file = Filesystem::fileGetAll($file);
                } else {
                    $file = $websitePageCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . SONICX_CACHE_SITE_NAME . SONICX_CACHE_SITE_HTML_EXTENSION;
                    return $file = Filesystem::fileGetAll($file);
                }
            } catch (\Exception $e) {
                return null;
            }
        } else {
            return null;
        }
    }

    public function buildPageCache($url)
    {
        try {
            $configuration = Context::getContext()->getConfiguration();
            $websiteId = $configuration[SONICX_WEBSITE_ID_KEY];
            $websiteSecret = $configuration[SONICX_WEBSITE_SECRET_KEY];
            if (empty($url)) {
                $responseData = HTTPClient::sendPOSTRequest(SONICX_WEBSITE_OPTIMIZATION_BULK_URL, [SONICX_URL_KEY => $url, SONICX_WEBSITE_ID_KEY => $websiteId, SONICX_WEBSITE_SECRET_KEY => $websiteSecret]);
                $this->rebuildPageCache($responseData);
            } else {
                $responseData = HTTPClient::sendPOSTRequest(SONICX_WEBSITE_OPTIMIZATION_URL, [SONICX_URL_KEY => $url, SONICX_WEBSITE_ID_KEY => $websiteId, SONICX_WEBSITE_SECRET_KEY => $websiteSecret]);
                $this->rebuildPageCache($responseData);
            }
            return true;
        } catch (\Exception $e) {
            return false;
        }
    }

    public function purgeCacheLocalCache($url)
    {
        $configuration = Context::getContext()->getConfiguration();
        $siteId = $configuration[SONICX_WEBSITE_ID_KEY];
        $websiteCacheFolder = SONICX_CACHE_PATH . SONICX_CACHE_FOLDER_DELIMITER . $siteId;
        if (is_dir(SONICX_CACHE_PATH) && is_dir($websiteCacheFolder) && $websiteCacheFolder) {
            if (empty($url)) {
                Filesystem::deleteDir($websiteCacheFolder);
                return true;
            } else {
                $websitePageCacheFolder = $websiteCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . md5($url);
                $file = $websitePageCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . SONICX_CACHE_SITE_NAME . SONICX_CACHE_SITE_HTML_EXTENSION;
                if (is_dir($websitePageCacheFolder) && is_file($file)) {
                    Filesystem::deleteFile($file);
                    Filesystem::deleteDir($websitePageCacheFolder);
                }
                return true;
            }
        } else {
            return true;
        }
    }

    public function purgeCache($url, $activityId)
    {
        $success = $this->purgeServerCache($url, $activityId);
        if ($success) {
            $configuration = Context::getContext()->getConfiguration();
            $siteId = $configuration[SONICX_WEBSITE_ID_KEY];
            $websiteCacheFolder = SONICX_CACHE_PATH . SONICX_CACHE_FOLDER_DELIMITER . $siteId;
            if (is_dir(SONICX_CACHE_PATH) && is_dir($websiteCacheFolder) && $websiteCacheFolder) {
                if (empty($url)) {
                    Filesystem::deleteDir($websiteCacheFolder);
                    return true;
                } else {
                    $websitePageCacheFolder = $websiteCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . md5($url);
                    $file = $websitePageCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . SONICX_CACHE_SITE_NAME . SONICX_CACHE_SITE_HTML_EXTENSION;
                    if (is_dir($websitePageCacheFolder) && is_file($file)) {
                        Filesystem::deleteFile($file);
                        Filesystem::deleteDir($websitePageCacheFolder);
                    }
                    return true;
                }
            } else {
                return true;
            }
        } else {
            return false;
        }
    }

    private function isCacheValid($siteId, $url, $userAgent)
    {
        try {
            $device = new Device($userAgent);
            $websiteCacheFolder = SONICX_CACHE_PATH . SONICX_CACHE_FOLDER_DELIMITER . $siteId;
            $websitePageCacheFolder = $websiteCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . md5($url);
            if (is_dir(SONICX_CACHE_PATH) && is_dir($websiteCacheFolder) && $websiteCacheFolder) {
                $siteName = $device->getType() == DeviceType::MOBILE ? SONICX_CACHE_MOBILE_SITE_NAME : SONICX_CACHE_SITE_NAME;
                $file = $websitePageCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . $siteName . SONICX_CACHE_SITE_HTML_EXTENSION;
                $headers = Filesystem::fileGetHeaders($file);
                if (empty($headers[SONICX_CACHE_EXPIRY_HEADER_KEY]) || time() > (int)$headers[SONICX_CACHE_EXPIRY_HEADER_KEY]) {
                    return false;
                } else {
                    return true;
                }
            } else {
                return false;
            }
        } catch (\Exception $e) {
            return false;
        }
    }

    private function createCacheHolderIfNotExists()
    {
        $configuration = Context::getContext()->getConfiguration();
        $siteId = $configuration[SONICX_WEBSITE_ID_KEY];
        if (is_dir(SONICX_CACHE_PATH) || @mkdir(SONICX_CACHE_PATH, SONICX_CACHE_FOLDER_PERMISSION, true)) {
            $websiteCacheFolder = SONICX_CACHE_PATH . SONICX_CACHE_FOLDER_DELIMITER . $siteId;
            if (is_dir($websiteCacheFolder) || @mkdir($websiteCacheFolder, SONICX_CACHE_FOLDER_PERMISSION, true)) {
                return true;
            }
        }
        throw new RuntimeException();
    }

    public function buildOrUpdatePageCache($url, $optimizedURL, $deviceType)
    {
        $configuration = Context::getContext()->getConfiguration();
        $cacheTTL = $configuration[SONICX_CACHE_TTL_KEY];
        $siteId = $configuration[SONICX_WEBSITE_ID_KEY];
        $websiteCacheFolder = SONICX_CACHE_PATH . SONICX_CACHE_FOLDER_DELIMITER . $siteId;
        $websitePageCacheFolder = $websiteCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . md5($url);
        if (is_dir($websitePageCacheFolder) || @mkdir($websitePageCacheFolder, SONICX_CACHE_FOLDER_PERMISSION, true)) {
            $response = wp_remote_get($optimizedURL);
            if ( ( !is_wp_error($response)) && (200 === wp_remote_retrieve_response_code( $response ) ) ) {
                $body = $response[SONICX_WEBSITE_OPTIMIZED_BODY_KEY];
                $headers = array();
                $headers[] = SONICX_CACHE_EXPIRY_HEADER_KEY . ":" . (time() + $cacheTTL);
                $siteName = $deviceType == DeviceType::MOBILE ? SONICX_CACHE_MOBILE_SITE_NAME : SONICX_CACHE_SITE_NAME;
                return Filesystem::filePutContents($websitePageCacheFolder . SONICX_CACHE_FOLDER_DELIMITER . $siteName . SONICX_CACHE_SITE_HTML_EXTENSION, $body, $headers);
            }
        }
        return false;
    }

    private function isValidURL()
    {
    }

    private function purgeServerCache($url, $activityId)
    {
        $configuration = Context::getContext()->getConfiguration();
        $websiteId = $configuration[SONICX_WEBSITE_ID_KEY];
        $websiteSecret = $configuration[SONICX_WEBSITE_SECRET_KEY];
        $body = [];
        if (empty($url)) {
            $body = [SONICX_WEBSITE_ID_KEY => $websiteId, SONICX_WEBSITE_SECRET_KEY => $websiteSecret];
        } else {
            $body = [SONICX_WEBSITE_URL_KEY => $url, SONICX_WEBPAGE_ID_KEY => $activityId, SONICX_WEBSITE_ID_KEY => $websiteId, SONICX_WEBSITE_SECRET_KEY => $websiteSecret];
        }
        return HTTPClient::sendDELETERequest(SONICX_WEBSITE_ACTIVITY_PURGE_URL, $body);
    }

    private function rebuildPageCache($optimizationData)
    {
        try {
            $url = $optimizationData["url"];
            $optimizedUrl = $optimizationData["desktopOptimizedUrl"];
            if (!empty($optimizedUrl)) {
                try {
                    $this->createCacheHolderIfNotExists();
                    $this->buildOrUpdatePageCache($url, $optimizedUrl, DeviceType::DESKTOP);
                } catch (\Exception $e) {
                    return false;
                }
            }
            $optimizedUrl = $optimizationData["mobileOptimizedUrl"];
            if (!empty($optimizedUrl)) {
                try {
                    $this->createCacheHolderIfNotExists();
                    $this->buildOrUpdatePageCache($url, $optimizedUrl, DeviceType::MOBILE);
                } catch (\Exception $e) {
                    return false;
                }
            }
            return true;
        } catch (Throwable $ex) {
            return false;
        }
    }

}