<?php
namespace Sonicx\Core\REST;
use PHPMailer\PHPMailer\Exception;

$sonicx_basePath = dirname(__FILE__) . '/../';
require_once $sonicx_basePath . 'vendor/autoload.php';
require_once $sonicx_basePath . 'includes/constant.php';


class HTTPClient {
    public static function send_post_request_async($url, $body, $callback, array $headers = []) {
        try {
            $client = new Client([SONICX_TIMEOUT_KEY => 3]);
            $promise = $client->requestAsync(SONICX_POST_REQUEST_METHOD, $url, array(SONICX_POST_JSON_KEY => $body, SONICX_REQUEST_HEADERS_KEY => array(SONICX_CONTENT_TYPE_KEY => SONICX_CONTENT_TYPE_APPLICATION_JSON)));
            $promise->then(function (ResponseInterface $res) use ($callback){
                $callback(json_decode($res->getBody())->data);
            }, function (RequestException $e){
            });
            $promises = array($promise);
            $results = Promise\settle($promises)->wait();
            return true;
        } catch (Exception $ex) {
            return false;
        }
    }

    public static function sendGETRequest($url, $paramsArray, array $headers = []) {
        try {
            $res = curl_init();
            $params = http_build_query($paramsArray);
            $getUrl = $url."?".$params;

            curl_setopt($res, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($res, CURLOPT_FOLLOWLOCATION, TRUE);
            curl_setopt($res, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($res, CURLOPT_URL, $getUrl);
            curl_setopt($res, CURLOPT_TIMEOUT, 30);

            $response = curl_exec($res);
            if($response === FALSE){
                return array();
            }
            $responseData = json_decode($response, TRUE);
            curl_close($res);
            return $responseData["data"];
        } catch (\Throwable $ex) {
            return array();
        }
    }

    public static function sendPOSTRequest($url, $body, array $headers = []) {
        try {
            $res = curl_init($url);
            curl_setopt_array($res, array(
                CURLOPT_POST => TRUE,
                CURLOPT_RETURNTRANSFER => TRUE,
                CURLOPT_SSL_VERIFYPEER => FALSE,
                CURLOPT_POSTFIELDS => json_encode($body),
                CURLOPT_HTTPHEADER => array(
                    "Content-Type: application/json"
                )
            ));
            $response = curl_exec($res);
            if($response === FALSE){
                return array();
            }
            $responseData = json_decode($response, TRUE);
            curl_close($res);
            return $responseData["data"];
        } catch (\Throwable $ex) {
            return array();
        }
    }

    public static function sendDELETERequest($url, $body, array $headers = []) {
        try {
            $res = curl_init($url);
            curl_setopt_array($res, array(
                CURLOPT_CUSTOMREQUEST => "DELETE",
                CURLOPT_RETURNTRANSFER => TRUE,
                CURLOPT_SSL_VERIFYPEER => FALSE,
                CURLOPT_POSTFIELDS => json_encode($body),
                CURLOPT_HTTPHEADER => array(
                    "Content-Type: application/json"
                )
            ));
            $response = curl_exec($res);
            if($response === FALSE){
                return array();
            }
            $responseData = json_decode($response, TRUE);
            curl_close($res);
            return true;
        } catch (\Throwable $ex) {
            return false;
        }
    }
}