<?php
$sonicx_basePath = dirname(__FILE__) . '/../';
require_once $sonicx_basePath . 'includes/CacheManager.php';
require_once $sonicx_basePath . 'includes/utility.php';

use Sonicx\Core\Cache\Cache;


function registerCachePurgeHook()
{
    add_action( 'wp_ajax_invalidate_page_cache', 'invalidatePageCache' );
}

function registerCacheRefreshHook()
{
    add_action( 'wp_ajax_warmup_page_cache', 'warmupPageCache' );
}

function registerCacheSynchronizeHook()
{
    add_action( 'wp_ajax_synchronize_page_cache', 'synchronizePageCache' );
}


function invalidatePageCache()
{
    $url = sanitize_input_post(SONICX_WEBSITE_URL_KEY);
    $activityId = sanitize_input_post(SONICX_ACTIVITY_ID_KEY);
    $actionType = sanitize_input_post(SONICX_CACHE_CLEAN_ACTION_TYPE);
    $cache = new Cache();
    if($actionType == 'local') {
        $success = $cache->purgeCacheLocalCache($url);
        if($success) {
            $data = ["Success" => true];
            return new WP_REST_Response( $data, 200 );
        } else {
            return new WP_Error( 500, __('Failed' ), array( 'message' => 'Purge failed. Please try again or contact support if problem persists.' ) );
        }
    } else {
        $success = $cache->purgeCache($url, $activityId);
        if($success) {
            $data = ["Success" => true];
            return new WP_REST_Response( $data, 200 );
        } else {
            return new WP_Error( 500, __('Failed' ), array( 'message' => 'Purge failed. Please try again or contact support if problem persists.' ) );
        }
    }
}

function warmupPageCache()
{
    $url = sanitize_input_post(SONICX_WEBSITE_URL_KEY);
    $cache = new Cache();
    $success = $cache->buildPageCache($url);
    if($success) {
        $success = synchronizePageCache();
        if($success) {
            $data = ["Success" => true];
            return new WP_REST_Response( $data, 200 );
        } else {
            return new WP_Error( 500, __('Failed' ), array( 'message' => 'Cache synchronization failed. Please try again or contact support if problem persists.' ) );
        }
        $data = ["Success" => true];
        return new WP_REST_Response( $data, 200 );
    } else {
        return new WP_Error( 500, __('Failed' ), array( 'message' => 'Purge failed. Please try again or contact support if problem persists.' ) );
    }
}

function synchronizePageCache()
{
    $cache = new Cache();
    return $cache->synchronizeCache();
}


function refreshPageCache($post_id)
{
    $url = get_permalink($post_id);
    if(!empty($url)) {
        $cache = new Cache();
        $success = $cache->purgeCache($url, null);
        if($success) {
            $success = $cache->buildPageCache($url);
            if($success) {
                $data = ["Success" => true];
                return new WP_REST_Response( $data, 200 );
            } else {
                return new WP_Error( 500, __('Failed' ), array( 'message' => 'Cache refresh. Please try again or contact support if problem persists.' ) );
            }
        } else {
            return new WP_Error( 500, __('Failed' ), array( 'message' => 'Cache refresh. Please try again or contact support if problem persists.' ) );
        }
    }
    return new WP_Error( 500, __('Failed' ), array( 'message' => 'Cache refresh. Please try again or contact support if problem persists.' ) );
}