<?php
/**
 * Breaking News Option Page
 *
 * @package     SX_Leospeed/Option
 * @author      Manpreet Singh
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Breaking-News Cart class
 *
 * create the option page
 */
class SX_Leospeed_Option
{

    /**
     * Holds the values to be used in the fields callbacks
     */
    private $options;

    /**
     * SX_Leospeed_Option constructor.
     */
    public function __construct()
    {
        // Admin menu for breaking news option page
        add_action('admin_menu', array($this, 'sx_plugin_page'));

        //Register and add settings
        add_action('admin_init', array($this, 'sx_settings_options'));

    }


    /**
     * Add a new option page for breaking news options
     *
     * @access public
     * @return void
     */
    public function sx_plugin_page()
    {

        $active = get_option(SX_Leospeed::$sx_option_active);

        if ($active == 'success') {
            add_menu_page(__('Leospeed', SX_Leospeed::$plugin_name), __('Leospeed', SX_Leospeed::$plugin_name), 'manage_options', 'leospeed-dashboard', array($this, 'leospeed_admin_menu_page_view'), SX_Leospeed::$plugin_url . 'public/images/leo.png', 1);
        }
        // This page will be under "Settings"
        add_options_page(
            __('Settings Admin', SX_Leospeed::$plugin_name),
            __('Leospeed Settings', SX_Leospeed::$plugin_name),
            'manage_options',
            'sx-setting-admin',
            array($this, 'create_admin_page')
        );
    }

    public function leospeed_admin_menu_page_view()
    {
        require_once(SX_Leospeed::$plugin_path . 'templates/container.php');
        require_once(SX_Leospeed::$plugin_path . 'templates/script.php');
        require_once(SX_Leospeed::$plugin_path . 'templates/dashboard.php');
        require_once(SX_Leospeed::$plugin_path . 'templates/optimization.php');
        require_once(SX_Leospeed::$plugin_path . 'templates/setting.php');
    }

    /**
     * Create admin breaking news option page.
     *
     * @access public
     * @return void
     */
    public function create_admin_page()
    {
        $active = get_option(SX_Leospeed::$sx_option_active);

        if ($active == 'success') {
            echo '<div class="notice notice-success is-dismissible">
	             <p>' . __('Leospeed connected successfully!', SX_Leospeed::$plugin_name) . '</p>
	         </div>';
        }
        // Set option property
        $this->options = get_option(SX_Leospeed::$sx_option_name);
        ?>
        <div class="wrap">
            <h1 style="text-align: center;"><img
                        src="<?php echo esc_url(SX_Leospeed::$plugin_url . 'public/images/leo.svg'); ?>"
                        alt="Leospeed Logo" style=" width: 70px; "/></h1>
            <div class="leospeed-setting-title-container">
                <div class="leospeed-setting-title">
                    <?php echo SX_Leospeed::$display_name; ?>
                </div>
            </div>
            <div style="text-align: center; font-size: 15px!important; font-weight: 600; margin: 10px 0px !important;">
                Please checkout the quick start guide here. For questions/suggestions/concerns email at <a
                        href="mailto:admin@leospeed.io?subject=Questions on Leospeed for WordPress">admin@leospeed.io</a>.
            </div>
            <div style="min-width: 100%; max-width: 100%">
                <div class="leospeed-setting-container" style="left: 25% !important; position: relative">
                    <div class="leospeed-setting-section">
                        <form method="post" action="options.php">
                            <div style=" text-align: center; width: 500px; margin: 0 auto; padding: 10px 0; ">
                                <?php
                                // This prints out all hidden setting fields
                                settings_fields('sx_option_group');
                                do_settings_sections('sx-setting-admin');
                                submit_button();
                                ?>
                            </div>
                        </form>
                    </div>
                </div>
            </div>
            <style type="text/css">
                p.submit {
                    text-align: center;
                    margin-top: -10px;
                }

                p.submit #submit {
                    background: -webkit-linear-gradient(left, #fe8464 0%, #fe6e9a 50%, #fe8464 100%);
                    background: -o-linear-gradient(left, #fe8464 0%, #fe6e9a 50%, #fe8464 100%);
                    background: linear-gradient(to right, #fe8464 0%, #fe6e9a 50%, #fe8464 100%);
                    border: 1px solid #fe8464;
                }

                p.submit #submit:focus {
                    box-shadow: 0 0 0 1px #fff, 0 0 0 3px #fe8464;
                }

                div.notice {
                    width: 500px;
                    margin: 10px auto !important;
                }

                h2 {
                    line-height: 45px;
                    background: -webkit-linear-gradient(left, #fe8464 0%, #fe6e9a 50%, #fe8464 100%);
                    background: -o-linear-gradient(left, #fe8464 0%, #fe6e9a 50%, #fe8464 100%);
                    background: linear-gradient(to right, #fe8464 0%, #fe6e9a 50%, #fe8464 100%);
                    background-size: 200%;
                    color: #fff;
                }

                .leospeed-setting-title-container {
                    justify-content: center !important;
                    display: flex;
                    margin: 0px 0px 10px 0px !important;
                }

                .leospeed-setting-title {
                    line-height: 1.42857143;
                    color: rgb(85, 85, 85);
                    font-weight: 500;
                    width: 200px !important;
                    font-size: 20px !important;
                    text-align: center;
                    background-color: rgb(255, 255, 255);
                    -webkit-box-shadow: 0 4px 3px rgba(0, 0, 0, 0.075);
                    box-shadow: 0 4px 3px rgba(0, 0, 0, 0.075);
                    -webkit-transition: border-color ease-in-out .15s, box-shadow ease-in-out .15s;
                    transition: border-color ease-in-out .15s, box-shadow ease-in-out .15s;
                }

                .leospeed-setting-container {
                    display: flex;
                    width: 550px !important;
                    line-height: 1.42857143;
                    background-color: rgb(255, 255, 255);
                    -webkit-box-shadow: 0 4px 3px rgba(0, 0, 0, 0.075);
                    box-shadow: 0 4px 3px rgba(0, 0, 0, 0.075);
                }

                .leospeed-setting-section {
                    width: 95%;
                    float: left;
                    position: relative;
                    left: 2%;
                }
            </style>
        </div>
        <?php
        if ((isset($this->options['sx_setting_update']) && $this->options['sx_setting_update'] == "1")
            && (isset($this->options['sx_setting_website_id']) && $this->options['sx_setting_website_id'] != "")
            && (isset($this->options['sx_setting_website_secret']) && $this->options['sx_setting_website_secret'] != "")
            && (isset($this->options['sx_setting_website_url']) && $this->options['sx_setting_website_url'] != "")
        ) {
            $this->options['sx_setting_update'] = '';
            update_option(SX_Leospeed::$sx_option_name, $this->options);
            update_option(SX_Leospeed::$sx_option_active, 'error');

            $endpoint = 'http://api.engine.lamborghini.botos.dev/website/activation';
            $website_id = $this->options['sx_setting_website_id'];
            $website_secret = $this->options['sx_setting_website_secret'];
            $website_url = $this->options['sx_setting_website_url'];
            $body = [
                "websiteId" => $website_id,
                "websiteSecret" => $website_secret,
                "url" => $website_url
            ];

            $body = wp_json_encode($body);

            $options = [
                'body' => $body,
                'headers' => [
                    'Content-Type' => 'application/json',
                ],
                'timeout' => 60,
                'redirection' => 5,
                'blocking' => true,
                'httpversion' => '1.0',
                'sslverify' => false,
                'data_format' => 'body',
            ];

            $response = wp_remote_post($endpoint, $options);
            if (is_wp_error($response)) {
                $error_message = $response->get_error_message();
                echo '<div class="notice notice-error is-dismissible">
		             <p>' . __('Something went wrong:', SX_Leospeed::$plugin_name) . $error_message . '</p>
		        </div>';
            } else {
                $result = json_decode($response['body'], true);
                if ($result['success'] == true) {
                    echo '<div class="notice notice-success is-dismissible">
			             <p>' . __('Leospeed connected successfully!', SX_Leospeed::$plugin_name) . '</p>
			         </div>';
                    update_option(SX_Leospeed::$sx_option_active, 'success');

                    echo '<script type="text/javascript">';
                    echo ' var sonicx_dashboard = "' . admin_url('options-general.php?page=leospeed-dashboard') . '";';
                    echo 'location.href = sonicx_dashboard;';
                    echo '</script>';
                } else {
                    echo '<div class="notice notice-error is-dismissible">
			             <p>' . __('Something went wrong: Leospeed is not connected.', SX_Leospeed::$plugin_name) . '</p>
			         </div>';
                }
            }
        }
    }

    /**
     * Register and add settings
     *
     * @access public
     * @return void
     */
    public function sx_settings_options()
    {

        // Registering the Sonicx option group

        register_setting(
            'sx_option_group',
            SX_Leospeed::$sx_option_name,
            array($this, 'sanitize')
        );

        // Option setting section info
        add_settings_section(
            'sx_setting_section',
            __('Existing User', SX_Leospeed::$plugin_name),
            array($this, 'sx_print_section_info'),
            'sx-setting-admin'
        );

        // Website ID option setting field
        add_settings_field(
            'sx_setting_website_id',
            __('Website ID', SX_Leospeed::$plugin_name),
            array($this, 'sx_setting_website_id_callback'),
            'sx-setting-admin',
            'sx_setting_section'
        );

        // Website Secret option setting field
        add_settings_field(
            'sx_setting_website_secret',
            __('Website Secret', SX_Leospeed::$plugin_name),
            array($this, 'sx_setting_website_secret_callback'),
            'sx-setting-admin',
            'sx_setting_section'
        );

        // Website URL option setting field
        add_settings_field(
            'sx_setting_website_url',
            __('Website URL', SX_Leospeed::$plugin_name),
            array($this, 'sx_setting_website_url_callback'),
            'sx-setting-admin',
            'sx_setting_section'
        );

        // Website URL option setting update field
        add_settings_field(
            'sx_setting_update',
            __('', SX_Leospeed::$plugin_name),
            array($this, 'sx_setting_update_callback'),
            'sx-setting-admin',
            'sx_setting_section'
        );

    }


    /**
     * Sanitize each setting field as needed
     *
     * @param $input
     *
     * @return array
     */
    public function sanitize($input)
    {
        $new_input = array();

        // Senitize the website id field
        if (isset($input['sx_setting_website_id'])
            && $input['sx_setting_website_id'] !== ""
        ) {
            $new_input['sx_setting_website_id'] = stripslashes(sanitize_text_field($input['sx_setting_website_id']));
        } else {
            // Error if the website id field is empty
            add_settings_error(
                'sx_setting_website_id',
                'sx_errors',
                __('Website ID field is required.', SX_Leospeed::$plugin_name),
                'error'
            );
        }

        // Senitize the website secret field
        if (isset($input['sx_setting_website_secret'])
            && $input['sx_setting_website_secret'] !== ""
        ) {
            $new_input['sx_setting_website_secret'] = stripslashes(sanitize_text_field($input['sx_setting_website_secret']));
        } else {
            // Error if the website secret field is empty
            add_settings_error(
                'sx_setting_website_secret',
                'sx_errors',
                __('Website Secret field is required.', SX_Leospeed::$plugin_name),
                'error'
            );
        }

        // Senitize the website url field
        if (isset($input['sx_setting_website_url'])
            && $input['sx_setting_website_url'] !== ""
        ) {
            $new_input['sx_setting_website_url'] = stripslashes(sanitize_url($input['sx_setting_website_url']));
        } else {
            // Error if the website url field is empty
            add_settings_error(
                'sx_setting_website_url',
                'sx_errors',
                __('Website URL field is required.', SX_Leospeed::$plugin_name),
                'error'
            );
        }

        if (isset($input['sx_setting_update']) && $input['sx_setting_update']) {
            $new_input['sx_setting_update'] = $input['sx_setting_update'];
        }

        return $new_input;
    }


    /**
     *
     *
     * @access public
     * @return void
     */
    public function sx_print_section_info()
    {
        print __('<div style="font-size: 15px;">In order to connect Leospeed below you need enter the following details. More information about these values can be found <a href="https://leospeed.io/download" target="_blank">here <i class="fa fa-external-link"></i></a></div>', SX_Leospeed::$plugin_name);
    }


    /**
     * Get the settings option array and print website id setting
     *
     * @access public
     * @return void
     */
    public function sx_setting_website_id_callback()
    {
        printf(
            '<input type="text" class="regular-text" id="sx_setting_website_id" name="%s[sx_setting_website_id]" value="%s" />',
            SX_Leospeed::$sx_option_name,
            isset($this->options['sx_setting_website_id']) ? esc_html($this->options['sx_setting_website_id']) : ''
        );
    }

    /**
     * Get the settings option array and website secret setting
     *
     * @access public
     * @return void
     */
    public function sx_setting_website_secret_callback()
    {
        printf(
            '<input type="text" class="regular-text" id="sx_setting_website_secret" name="%s[sx_setting_website_secret]" value="%s" />',
            SX_Leospeed::$sx_option_name,
            isset($this->options['sx_setting_website_secret']) ? esc_html($this->options['sx_setting_website_secret']) : ''
        );
    }

    /**
     * Get the settings option array and website url setting
     *
     * @access public
     * @return void
     */
    public function sx_setting_website_url_callback()
    {
        printf(
            '<input type="text" class="regular-text" id="sx_setting_website_url" name="%s[sx_setting_website_url]" value="%s" />',
            SX_Leospeed::$sx_option_name,
            isset($this->options['sx_setting_website_url']) ? esc_html($this->options['sx_setting_website_url']) : site_url('/')
        );
    }

    /**
     * Get the settings option array and update setting
     *
     * @access public
     * @return void
     */
    public function sx_setting_update_callback()
    {
        printf(
            '<input type="hidden" class="regular-text" id="sx_setting_update" name="%s[sx_setting_update]" value="%s" />',
            SX_Leospeed::$sx_option_name,
            1
        );
    }
} // end \SX_Leospeed_Option class
