<?php
$sonicx_basePath = dirname(__FILE__) . '/../';
require_once $sonicx_basePath . 'includes/constant.php';
require_once $sonicx_basePath . 'includes/CacheManager.php';
require_once $sonicx_basePath . 'includes/HTTPClient.php';
require_once $sonicx_basePath . 'includes/DeviceType.php';
require_once $sonicx_basePath . 'includes/cache.php';
require_once $sonicx_basePath . 'includes/utility.php';


use PHPMailer\PHPMailer\Exception;
use Sonicx\Core\Cache\Cache;
use Sonicx\Core\REST\HTTPClient;
use Sonicx\Core\Device\DeviceType;

function handle_client_request()
{
    if (!empty($_GET['xSonicxRequest']) && !empty($_GET['xSonicxAction'])) {
        handleAction($_GET);
    } else if (!empty($_GET['xSonicxRequest']) && empty($_GET['xSonicxAction'])) {
    } else if (!((!empty($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME'] == 'https') ||
        (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ||
        (!empty($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == '443'))) {
    } else if (!empty($_SERVER['X-SONICX-SOURCE']) && $_SERVER['X-SONICX-SOURCE'] == 'CDN') {
    } else {
        try {
            if (isCacheAllowed()) {
                $options = get_option('leospeed_plugin_option_settings');

                if ((isset($options['sx_setting_website_id']) && $options['sx_setting_website_id'] != "")
                    && (isset($options['sx_setting_website_secret']) && $options['sx_setting_website_secret'] != "")
                ) {
                    $websiteId = $options['sx_setting_website_id'];
                    $websiteSecret = $options['sx_setting_website_secret'];

                    $cache = new Cache();
                    $userAgent = !empty(sanitize_string($_SERVER['HTTP_USER_AGENT'])) ? sanitize_string($_SERVER['HTTP_USER_AGENT']) : 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36';
                    $cacheContent = $cache->getPageCache(home_url($_SERVER["REQUEST_URI"]), $userAgent);
                    if ($cacheContent != null) {
                        echo $cacheContent->content;
                        exit;
                    } else {
                        $cache->buildPageCache(home_url($_SERVER[SONICX_REQUEST_URI_KEY]));
                    }
                }
            }
        } catch (Throwable $ex) {

        }
    }
}

function handleAction($actionData)
{
    $action = $actionData['xSonicxAction'];
    if (!empty($action) && $action == "delete") {
        $url = $actionData['url'];
        $activityId = $actionData['activityId'];
        $cache = new Cache();
        $success = $cache->purgeCache($url, $activityId);
        if ($success) {
            $data = ["Success" => true];
            return new WP_REST_Response($data, 200);
        } else {
            return new WP_Error(500, __('Failed'), array('message' => 'Purge failed. Please try again or contact support if problem persists.'));
        }
    } elseif (!empty($action) && $action == "build") {
        $url = $actionData['url'];
        $optimizedURL = $actionData['optimizedUrl'];
        $deviceType = $actionData['platform'];
        $cache = new Cache();
        $cache->buildOrUpdatePageCache($url, $optimizedURL, $deviceType);
        $data = ["Success" => true];
        return new WP_REST_Response($data, 200);
    }
    return new WP_Error(500, __('Failed'), array('message' => 'Unsupported operation'));
}


function isCacheAllowed()
{
    if (!empty($_SERVER[SONICX_WARM_UP_REQUEST_HEADER])) {
        return false;
    }
    if ((isAJAXRequest()) || !(isRequestMethod("GET") || isRequestMethod("HEAD"))) {// TODO: Allow URLs which match a pattern in the AJAX URL whitelist
        return false; // don't cache ajax or not GET requests
    }

    if (!empty($_SERVER["HTTP_USER_AGENT"]) && (preg_match("~MSIE|Internet Explorer~i", $_SERVER["HTTP_USER_AGENT"]) || strpos($_SERVER["HTTP_USER_AGENT"], "Trident/7.0; rv:11.0") !== false)) { // Skip IE
        return false;
    }
    $excludedCookies = array(array("name" => "wordpress_logged_in_*", "values" => array()), array("name" => "wp-postpass_*", "values" => array()), array("name" => "wptouch_switch_toggle", "values" => array()), array("name" => "comment_author_*", "values" => array()), array("name" => "comment_author_email_*", "values" => array()));
    foreach ($excludedCookies as $cookieExclude) {
        foreach (getCookies() as $cookieName => $cookieValue) {
            if (preg_match('/^' . wildcardToRegex($cookieExclude["name"]) . '$/', $cookieName)) {
                if (count($cookieExclude["values"]) == 0) {
                    return false; // no excluded cookie values entered, reject all values
                } else {
                    foreach ($cookieExclude["values"] as $val) {
                        if (preg_match('/^' . wildcardToRegex($val) . '$/', $cookieValue)) {
                            return false;
                        }
                    }
                }
            }
        }
    }
    return true;
}

function isRequestMethod($method)
{
    return empty($_SERVER['REQUEST_METHOD']) || $_SERVER['REQUEST_METHOD'] == $method;
}

function isAJAXRequest()
{
    return !empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest';
}

function getCookies()
{
    $cookies = array();
    foreach ($_COOKIE as $name => $value) {
        if (is_array($value)) {
            foreach ($value as $k => $v) {
                $key = $name . "[$k]";
                $cookies[sanitize_key($key)] = sanitize_text_field($v);
            }
        } else {
            $cookies[sanitize_key($name)] = sanitize_text_field($value);
        }
    }
    $cookieFilters = array();
    foreach ($cookieFilters as $cookieFilter) {
        call_user_func_array($cookieFilter, array(&$cookies));
    }

    return $cookies;
}

function wildcardToRegex($str, $delim = "/")
{
    return implode(".*?", array_map(function ($input) use ($delim) {
        return preg_quote($input, $delim);
    }, explode("*", $str)));
}

