<?php

/*
Plugin Name: Leospeed
Plugin URI: https://leospeed.io
Description: Leospeed improves your website's core web vitals score. It uses a combination for Google recommended optimizations with CDN to deliver best possible scores for your wordpress website at most affordable prices. Provide your website url, we generate demo for you in minutes, test the results in tools like Page Speed Insights, GTMetrix, Pingdom, if satisfied install the plugin.
Version: 2.0.2
Author: Leowand
License:      GPL2
License URI:  https://www.gnu.org/licenses/gpl-2.0.html
*/

$np_basePath = dirname(__FILE__) . '/';
require_once $np_basePath . 'includes/activation.php';


if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'SX_Leospeed' ) ) :
	/**
	 * Main Plugin Class
	 *
	 * @since 1.0
	 */
	class SX_Leospeed {

		/**
		 * Plugin name for text domains.
		 *
		 * @var $string
		 */
		public static $plugin_name = 'leospeed-plugin';

		/**
		 * Plugin base name
		 *
		 * @var string
		 */
		public static $plugin_basename;

		/**
		 * Plugin file path
		 *
		 * @var string
		 */
		public static $plugin_path;

		/**
		 * Plugin url
		 *
		 * @var string
		 */
		public static $plugin_url;

		/**
		 * Leospeed setting page option field
		 *
		 * @var string
		 */
		public static $sx_option_name = 'leospeed_plugin_option_settings';
		
		/**
		 * Leospeed setting page option field
		 *
		 * @var string
		 */
		public static $sx_option_active = 'leospeed_plugin_active';

		public static $display_name = 'Leospeed';

		public function __construct() {
			self::$plugin_basename = plugin_basename( __FILE__ );
			self::$plugin_path     = plugin_dir_path( __FILE__ );
			self::$plugin_url      = plugin_dir_url( __FILE__ );

			// load core classes
			$this->load_classes();

            add_action('admin_enqueue_scripts', array($this, 'admin_assets') );

			// Adding settings link to plugin page
			add_filter( 'plugin_action_links_' . plugin_basename( self::$plugin_basename ), array(
				$this,
				'plugin_settings_link'
			) );
			
//			add_action('rest_api_init', array( $this, 'register_hooks' ));

			add_action( 'init', function() {
			    if (current_user_can( 'manage_options' )) {
			        add_action('admin_bar_menu', function ($wp_admin_bar) {
			            $node = array(
			                'id' => 'leospeed-top-menu',
			                'title' => '&nbsp;&nbsp;<i style="" class="fa fa-circle nitro nitro-status nitro-status-ok" aria-hidden="true"></i>&nbsp;&nbsp;Leospeed',
			                'href' => admin_url( 'options-general.php?page=leospeed-dashboard' ),
			                'meta' => array(
			                    'class' => 'custom-node-class'
			                )
			            );
			            $wp_admin_bar->add_menu($node);
			        }, 50);
			    }
			});
		}
		public function register_hooks(){
			
		    register_cache_hooks();
		}

		public function admin_assets() {
		    $version = "1.1";
            $current_screen = get_current_screen();
            if($current_screen->base === 'toplevel_page_leospeed-dashboard') {
                wp_enqueue_style( 'leospeed_admin_setting_css', self::$plugin_url . 'public/css/style.css?leospeed_v='.$version );
                wp_enqueue_style( 'leospeed_angular_material_css', self::$plugin_url . 'public/css/angular-material.min.css?leospeed_v='.$version);
                wp_enqueue_style( 'leospeed_polaris_css', self::$plugin_url . 'public/css/polaris.min.css?leospeed_v='.$version);
            }

            wp_enqueue_script( 'leospeed_angular_script', self::$plugin_url . 'public/js/angular.js?leospeed_v='.$version);
            wp_enqueue_script( 'leospeed_angular_animate_script', self::$plugin_url . 'public/js/angular-animate.min.js?leospeed_v='.$version);
            wp_enqueue_script( 'leospeed_angular_aria_script', self::$plugin_url . 'public/js/angular-aria.min.js?leospeed_v='.$version);
            wp_enqueue_script( 'leospeed_angular_material_script', self::$plugin_url . 'public/js/angular-material.min.js?leospeed_v='.$version);
            wp_enqueue_script( 'leospeed_angular_messages_script', self::$plugin_url . 'public/js/angular-messages.min.js?leospeed_v='.$version);
            wp_enqueue_script( 'jquery');
            wp_enqueue_script( 'leospeed_admin_controller_script', self::$plugin_url . 'public/js/activation.controller.php?leospeed_v='.$version);
            wp_enqueue_script( 'page-cache-script', self::$plugin_url . 'public/js/dashboard.controller.js?leospeed_v='.$version, array('jquery'));
            wp_localize_script(
                'page-cache-script',
                'invalidate_page_cache_handler',
                array(
                    'ajaxurl' => admin_url('admin-ajax.php'),
                    'nonce' => wp_create_nonce('ajax-nonce')
                )
            );
            wp_localize_script(
                'page-cache-script',
                'warmup_page_cache_handler',
                array(
                    'ajaxurl' => admin_url('admin-ajax.php'),
                    'nonce' => wp_create_nonce('ajax-nonce')
                )
            );
            wp_localize_script(
                'page-cache-script',
                'synchronize_page_cache_handler',
                array(
                    'ajaxurl' => admin_url('admin-ajax.php'),
                    'nonce' => wp_create_nonce('ajax-nonce')
                )
            );
		}

		/**
		 * Load plugin classes
		 *
		 * @access public
		 * @return void
		 */
		public function load_classes() {

//				require_once( 'service/admin/setting.php' );
				require_once( 'includes/view.php' );
                require_once( 'includes/activation.php' );
				require_once( 'includes/webhook.php' );
				require_once( 'includes/cache.php' );
							if ( is_admin() ) {

				// load option page class
				require_once( 'includes/option-page.php' );
				$this->options = new SX_Leospeed_Option();
			}
		}

		/**
		 * Add setting shortcut link in plugin section
		 *
		 * @access public
		 *
		 * @param mixed $links
		 *
		 * @return void
		 */
		public function plugin_settings_link( $links ) {
			$settings_link = '<a href="' . admin_url( 'options-general.php?page=leospeed-dashboard' ) . '">' . __( 'Settings', self::$plugin_name ) . '</a>';
			array_unshift( $links, $settings_link );

			return $links;
		}
	}
endif;

add_action( 'plugins_loaded', 'leospeed_plugin_init' );
add_action( 'wp_ajax_invalidate_page_cache', 'invalidate_page_cache' );
add_action( 'wp_ajax_warmup_page_cache', 'warmup_page_cache' );
add_action( 'wp_ajax_synchronize_page_cache', 'synchronize_page_cache' );
add_action( 'save_post', 'refresh_cache_post_save' );
add_action( 'leospeed_system_task_hook', 'leospeed_system_task');

/**
 * Initializes the extension.
 *
 * @return Object Instance of the extension.
 */

function invalidate_page_cache()
{
    invalidatePageCache();
}

function warmup_page_cache()
{
    warmupPageCache();
}

function refresh_cache_post_save($post_id) {
    refreshPageCache($post_id);
}

function synchronize_page_cache()
{
    synchronizePageCache();
}


register_deactivation_hook( __FILE__, "leospeed_deactivate_website");
register_activation_hook( __FILE__, "leospeed_activate_website");

register_uninstall_hook( __FILE__, "leospeed_uninstall_website");

function leospeed_plugin_init() {
	// Plugin translation strings
	load_plugin_textdomain( 'leospeed-plugin', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );

	//Plugin global variable
	$GLOBALS['leospeed_plugin'] = new SX_Leospeed();

	if (!is_admin()) {
	    handle_client_request();
    }
}
