(function () {
    'use strict';

    angular
        .module('activation', ['ngMaterial'])
        .controller("ActivationController", ActivationController);

    ActivationController.$inject = ['$scope', '$http', '$window'];

    function ActivationController($scope, $http, $window) {
        $scope.websiteDetails = {
            websiteId: null,
            websiteSecret: null,
            url: null
        };

        $scope.activationStatus = {
            inProgress: false,
            isFailed: false,
            message: null
        }

        $scope.clearWebsiteUrl = function () {
            if (!$scope.websiteDetails.url.endsWith("/")) {
                $scope.websiteDetails.url = $scope.websiteDetails.url + "/";
            }
        };

        $scope.initializeWebsiteUrl = function (websiteUrl) {
            $scope.websiteDetails.url = websiteUrl;
            $scope.clearWebsiteUrl();
        }

        $scope.activateWebsite = function () {
            $scope.activationStatus.isFailed = false;
            $scope.activationStatus.inProgress = true;
            $scope.clearWebsiteUrl();
            $http({
                method: "POST",
                url: '<?php echo SONICX_ACCOUNT_ACTIVATION_URL; ?>',
                data: $scope.websiteDetails
            }).then(function (response, status) {
                $http({
                    method: "POST",
                    url: '<?php echo SONICX_ACCOUNT_ACTIVATION_CALLBACK_ENDPOINT; ?>',
                    data: {
                        websiteId: $scope.websiteDetails.websiteId,
                        websiteSecret: $scope.websiteDetails.websiteSecret
                    }
                }).then(function (response, status) {
                    $scope.activationStatus.inProgress = false;
                    $window.location.reload();
                }, function (response, status) {
                    $scope.activationStatus.isFailed = true;
                    $scope.activationStatus.inProgress = false;
                    $scope.activationStatus.message = "Website activation failed. Please try again or contact support if problem persists.";
                });
            }, function (response, status) {
                $scope.activationStatus.inProgress = false;
                $scope.activationStatus.isFailed = true;
                if(response.data) {
                    $scope.activationStatus.message = response.data.error.message;
                } else {
                    $scope.activationStatus.message = "Website activation failed. Please try again or contact support if problem persists.";
                }
            });
        }

        $scope.deactivateWebsite = function () {
            $scope.activationStatus.isFailed = false;
            $scope.activationStatus.inProgress = true;
            $scope.clearWebsiteUrl();
            $http({
                method: "DELETE",
                url: '<?php echo SONICX_ACCOUNT_ACTIVATION_URL; ?>',
                data: $scope.websiteDetails
            }).then(function (response, status) {
                $scope.activationStatus.inProgress = false;
                $scope.activationStatus.isFailed = true;
                $scope.activationStatus.message = "Website deactivation success.";
            }, function (response, status) {
                $scope.activationStatus.inProgress = false;
                $scope.activationStatus.isFailed = true;
                if(response.data) {
                    $scope.activationStatus.message = response.data.error.message;
                } else {
                    $scope.activationStatus.message = "Website activation failed. Please try again or contact support if problem persists.";
                }
            });
        }

    }
})();
